-- opb_sdram_w.vhd

library ieee;
use ieee.std_logic_1164.all;

library opb_sdram_v1_00_c;
use opb_sdram_v1_00_c.all;

ENTITY opb_sdram_w IS

    -- Declare wrapper generic parameters here
            generic (
                -- sdram generics
                C_INCLUDE_BURST_SUPPORT         : INTEGER  := 0;
                C_INCLUDE_HIGHSPEED_PIPE        : INTEGER  := 1;
                C_FAMILY                        : STRING  := "virtex2";
                C_SDRAM_TMRD                    : INTEGER  := 2;
                C_SDRAM_TWR                     : INTEGER  := 15000;
                C_SDRAM_TCCD                    : INTEGER  := 1;
                C_SDRAM_TRAS                    : INTEGER  := 40000;
                C_SDRAM_TRC                     : INTEGER  := 65000;
                C_SDRAM_TRFC                    : INTEGER  := 75000;
                C_SDRAM_TRCD                    : INTEGER  := 20000;
                C_SDRAM_TRRD                    : INTEGER  := 15000;
                C_SDRAM_TRP                     : INTEGER  := 20000;
                C_SDRAM_TREF                    : INTEGER  := 64;
                C_SDRAM_REFRESH_NUMROWS         : INTEGER  := 8192;
                C_SDRAM_CAS_LAT                 : INTEGER  := 2;
                C_SDRAM_DWIDTH                  : INTEGER  := 16;
                C_SDRAM_AWIDTH                  : INTEGER  := 13;
                C_SDRAM_COL_AWIDTH              : INTEGER  := 9;
                C_SDRAM_BANK_AWIDTH             : INTEGER  := 2;
        
                -- auto-calculated generic
                C_SDRAM_TREFI                   : INTEGER  := 7812500;
                
                -- ipif generics
                C_BASEADDR                      : STD_LOGIC_VECTOR := x"30000000";
                C_HIGHADDR                      : STD_LOGIC_VECTOR := x"3000ffff";
                C_OPB_DWIDTH                    : INTEGER := 32;
                C_OPB_AWIDTH                    : INTEGER := 32;
                C_OPB_CLK_PERIOD_PS             : INTEGER := 10000
             );  
        

    -- Declare wrapper ports here
          port
              (
               -- System Port Declarations ********************************************
        
               -- OPB Port Declarations ***********************************************
                OPB_ABus       : in std_logic_vector(0 to C_OPB_AWIDTH - 1 );
                OPB_DBus       : in std_logic_vector(0 to C_OPB_DWIDTH - 1 );
                Sln_DBus       : out std_logic_vector(0 to C_OPB_DWIDTH - 1 );
                OPB_select     : in std_logic := '0';
                OPB_RNW        : in std_logic := '0';
                OPB_seqAddr    : in std_logic := '0';
                OPB_BE         : in std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 );
                Sln_xferAck    : out std_logic;
                Sln_errAck     : out std_logic;
                Sln_toutSup    : out std_logic;
                Sln_retry      : out std_logic;
        
              
                -- SDRAM interface signals
                SDRAM_Clk             : out std_logic;
                SDRAM_CKE             : out std_logic;
                SDRAM_CSn             : out std_logic;
                SDRAM_RASn            : out std_logic;
                SDRAM_CASn            : out std_logic;
                SDRAM_WEn             : out std_logic;
                SDRAM_DQM             : out std_logic_vector(0 to C_SDRAM_DWIDTH/8-1);
                SDRAM_BankAddr        : out std_logic_vector(0 to C_SDRAM_BANK_AWIDTH-1);
                SDRAM_Addr            : out std_logic_vector(0 to C_SDRAM_AWIDTH-1);
                SDRAM_DQ_o            : out std_logic_vector(0 to C_SDRAM_DWIDTH-1);
                SDRAM_DQ_i            : in  std_logic_vector(0 to C_SDRAM_DWIDTH-1);
                SDRAM_DQ_t            : out std_logic_vector(0 to C_SDRAM_DWIDTH-1);
        
                -- Clocks and reset
                OPB_Clk             : in  std_logic;
                --Clk                 : out std_logic;
                --DCM_locked          : out std_logic;     
                SDRAM_Clk_in        : in  std_logic;
                --DCM_Rst             : in  std_logic;
                OPB_Rst             : in  std_logic                                        
              );                                
END ENTITY opb_sdram_w;

-- architecture implementation start here --

ARCHITECTURE implementation OF opb_sdram_w IS

  COMPONENT opb_sdram IS

    -- Declare generic parameters here
            generic (
                -- sdram generics
                C_INCLUDE_BURST_SUPPORT         : integer;
                C_INCLUDE_HIGHSPEED_PIPE        : integer;
                C_FAMILY                        : string;
                C_SDRAM_TMRD                    : integer;
                C_SDRAM_TWR                     : integer;
                C_SDRAM_TCCD                    : integer;
                C_SDRAM_TRAS                    : integer;
                C_SDRAM_TRC                     : integer;
                C_SDRAM_TRFC                    : integer;
                C_SDRAM_TRCD                    : integer;
                C_SDRAM_TRRD                    : integer;
                C_SDRAM_TRP                     : integer;
                C_SDRAM_TREF                    : integer;
                C_SDRAM_REFRESH_NUMROWS         : integer;
                C_SDRAM_CAS_LAT                 : integer;
                C_SDRAM_DWIDTH                  : integer;
                C_SDRAM_AWIDTH                  : integer;
                C_SDRAM_COL_AWIDTH              : integer;
                C_SDRAM_BANK_AWIDTH             : integer;
        
                -- auto-calculated generic
                C_SDRAM_TREFI                   : integer;
                
                -- ipif generics
                C_BASEADDR                      : std_logic_vector;
                C_HIGHADDR                      : std_logic_vector;
                C_OPB_DWIDTH                    : integer;
                C_OPB_AWIDTH                    : integer;
                C_OPB_CLK_PERIOD_PS             : integer
             );  
        

    -- Declare ports here
          port
              (
               -- System Port Declarations ********************************************
        
               -- OPB Port Declarations ***********************************************
                OPB_ABus       : in std_logic_vector(0 to C_OPB_AWIDTH - 1 );
                OPB_DBus       : in std_logic_vector(0 to C_OPB_DWIDTH - 1 );
                Sln_DBus       : out std_logic_vector(0 to C_OPB_DWIDTH - 1 );
                OPB_select     : in std_logic := '0';
                OPB_RNW        : in std_logic := '0';
                OPB_seqAddr    : in std_logic := '0';
                OPB_BE         : in std_logic_vector(0 to C_OPB_DWIDTH/8 - 1 );
                Sln_xferAck    : out std_logic;
                Sln_errAck     : out std_logic;
                Sln_toutSup    : out std_logic;
                Sln_retry      : out std_logic;
        
              
                -- SDRAM interface signals
                SDRAM_Clk             : out std_logic;
                SDRAM_CKE             : out std_logic;
                SDRAM_CSn             : out std_logic;
                SDRAM_RASn            : out std_logic;
                SDRAM_CASn            : out std_logic;
                SDRAM_WEn             : out std_logic;
                SDRAM_DQM             : out std_logic_vector(0 to C_SDRAM_DWIDTH/8-1);
                SDRAM_BankAddr        : out std_logic_vector(0 to C_SDRAM_BANK_AWIDTH-1);
                SDRAM_Addr            : out std_logic_vector(0 to C_SDRAM_AWIDTH-1);
                SDRAM_DQ_o            : out std_logic_vector(0 to C_SDRAM_DWIDTH-1);
                SDRAM_DQ_i            : in  std_logic_vector(0 to C_SDRAM_DWIDTH-1);
                SDRAM_DQ_t            : out std_logic_vector(0 to C_SDRAM_DWIDTH-1);
        
                -- Clocks and reset
                OPB_Clk             : in  std_logic;
                --Clk                 : out std_logic;
                --DCM_locked          : out std_logic;     
                SDRAM_Clk_in        : in  std_logic;
                --DCM_Rst             : in  std_logic;
                OPB_Rst             : in  std_logic                                        
              );                                
  END COMPONENT opb_sdram;

BEGIN -- architecture implementation

  opb_sdram_imp : opb_sdram

    GENERIC MAP (   -- Declare generic map here
            C_INCLUDE_BURST_SUPPORT     => C_INCLUDE_BURST_SUPPORT,
            C_INCLUDE_HIGHSPEED_PIPE    => C_INCLUDE_HIGHSPEED_PIPE,
            C_FAMILY                    => C_FAMILY,
            C_SDRAM_TMRD                => C_SDRAM_TMRD,
            C_SDRAM_TWR                 => C_SDRAM_TWR,
            C_SDRAM_TCCD                => C_SDRAM_TCCD,
            C_SDRAM_TRAS                => C_SDRAM_TRAS,
            C_SDRAM_TRC                 => C_SDRAM_TRC,
            C_SDRAM_TRFC                => C_SDRAM_TRFC,
            C_SDRAM_TRCD                => C_SDRAM_TRCD,
            C_SDRAM_TRRD                => C_SDRAM_TRRD,
            C_SDRAM_TRP                 => C_SDRAM_TRP,
            C_SDRAM_TREF                => C_SDRAM_TREF,
            C_SDRAM_REFRESH_NUMROWS     => C_SDRAM_REFRESH_NUMROWS,
            C_SDRAM_CAS_LAT             => C_SDRAM_CAS_LAT,
            C_SDRAM_DWIDTH              => C_SDRAM_DWIDTH,
            C_SDRAM_AWIDTH              => C_SDRAM_AWIDTH,
            C_SDRAM_COL_AWIDTH          => C_SDRAM_COL_AWIDTH,
            C_SDRAM_BANK_AWIDTH         => C_SDRAM_BANK_AWIDTH,
            C_SDRAM_TREFI               => C_SDRAM_TREFI,
            C_BASEADDR                  => C_BASEADDR,
            C_HIGHADDR                  => C_HIGHADDR,
            C_OPB_DWIDTH                => C_OPB_DWIDTH,
            C_OPB_AWIDTH                => C_OPB_AWIDTH,
            C_OPB_CLK_PERIOD_PS         => C_OPB_CLK_PERIOD_PS
                )

    PORT MAP (      -- Declare port map here
            OPB_ABus                    => OPB_ABus,
            OPB_DBus                    => OPB_DBus,
            Sln_DBus                    => Sln_DBus,
            OPB_select                  => OPB_select,
            OPB_RNW                     => OPB_RNW,
            OPB_seqAddr                 => OPB_seqAddr,
            OPB_BE                      => OPB_BE,
            Sln_xferAck                 => Sln_xferAck,
            Sln_errAck                  => Sln_errAck,
            Sln_toutSup                 => Sln_toutSup,
            Sln_retry                   => Sln_retry,
            SDRAM_Clk                   => SDRAM_Clk,
            SDRAM_CKE                   => SDRAM_CKE,
            SDRAM_CSn                   => SDRAM_CSn,
            SDRAM_RASn                  => SDRAM_RASn,
            SDRAM_CASn                  => SDRAM_CASn,
            SDRAM_WEn                   => SDRAM_WEn,
            SDRAM_DQM                   => SDRAM_DQM,
            SDRAM_BankAddr              => SDRAM_BankAddr,
            SDRAM_Addr                  => SDRAM_Addr,
            SDRAM_DQ_o                  => SDRAM_DQ_o,
            SDRAM_DQ_i                  => SDRAM_DQ_i,
            SDRAM_DQ_t                  => SDRAM_DQ_t,
            OPB_Clk                     => OPB_Clk,
            SDRAM_Clk_in                => SDRAM_Clk_in,
            OPB_Rst                     => OPB_Rst
            );

END ARCHITECTURE implementation;
